@@ -6,7 +6,7 @@
 #                      \___/_/\_\ .__/ \__,_|\__|
 #                               |_| XML parser
 #
-# Copyright (c) 2017-2021 Sebastian Pipping <sebastian@pipping.org>
+# Copyright (c) 2017-2023 Sebastian Pipping <sebastian@pipping.org>
 # Copyright (c) 2018      Marco Maggi <marco.maggi-ipsu@poste.it>
 # Copyright (c) 2019      Mohammed Khajapasha <mohammed.khajapasha@intel.com>
 # Licensed under the MIT license:
@@ -51,11 +51,9 @@ _get_build_dir() {
 
     local char_part=
     if ${unicode_enabled}; then
-        if ${with_unsigned_char}; then
-            char_part=__ushort
-        else
-            char_part=__wchar_t
-        fi
+        char_part=__wchar_t
+    elif ${with_unsigned_char}; then
+        char_part=__uchar
     else
         char_part=__char
     fi
@@ -70,7 +68,17 @@ _get_build_dir() {
         m32_part=__m32
     fi
 
-    echo "build__${version}__xml_context_${xml_context}${libbsd_part}${mingw_part}${char_part}${xml_attr_part}${m32_part}"
+    local ge_part=
+    if ${with_ge}; then
+        ge_part=__ge
+    fi
+
+    local dtd_part=
+    if ${with_dtd}; then
+        dtd_part=__dtd
+    fi
+
+    echo "build__${version}__xml_context_${xml_context}${libbsd_part}${mingw_part}${char_part}${ge_part}${dtd_part}${xml_attr_part}${m32_part}"
 }
 
 
@@ -97,6 +105,8 @@ _call_cmake() {
     ${with_libbsd} && cmake_args+=( -DEXPAT_WITH_LIBBSD=ON )
     ${with_mingw} && cmake_args+=( -DCMAKE_TOOLCHAIN_FILE="${abs_source_dir}"/cmake/mingw-toolchain.cmake )
     ${with_m32} && cmake_args+=( -D_EXPAT_M32=ON )
+    ${with_ge} || cmake_args+=( -DEXPAT_GE=OFF )
+    ${with_dtd} || cmake_args+=( -DEXPAT_DTD=OFF )
 
     (
         set -x
@@ -119,8 +129,8 @@ _copy_missing_mingw_libaries() {
     # * coverage GCC flags make them needed
     # * With WINEDLLPATH Wine looks for .dll.so in these folders, not .dll
     local target="$1"
-    local mingw_gcc_dll_dir="$(dirname "$(ls -1 /usr/lib*/gcc/i686-w64-mingw32/*/libgcc_s_sjlj-1.dll | head -n1)")"
-    for dll in libgcc_s_sjlj-1.dll libstdc++-6.dll; do
+    local mingw_gcc_dll_dir="$(dirname "$(ls -1 /usr/lib*/gcc/i686-w64-mingw32/*/{libgcc_s_sjlj-1.dll,libstdc++-6.dll} | head -n1)")"
+    for dll in libgcc_s_dw2-1.dll libgcc_s_sjlj-1.dll libstdc++-6.dll; do
         (
             set -x
             ln -s "${mingw_gcc_dll_dir}"/${dll} "${target}"/${dll}
@@ -139,7 +149,7 @@ _copy_missing_mingw_libaries() {
         done
     fi
 
-    for dll in libexpat{,w}.dll; do
+    for dll in libexpat{,w}-*.dll; do
         (
             set -x
             ln -s "${abs_build_dir}"/${dll} "${target}"/${dll}
@@ -161,7 +171,7 @@ _run() {
     ${with_unsigned_char} && BASE_FLAGS="${BASE_FLAGS} -funsigned-char"
 
     local CFLAGS="-std=c99 ${BASE_FLAGS}"
-    local CXXFLAGS="-std=c++98 ${BASE_FLAGS}"
+    local CXXFLAGS="-std=c++11 ${BASE_FLAGS}"
 
     (
         set -e
@@ -186,7 +196,10 @@ _run() {
         fi
 
         set -x
-        make CTEST_OUTPUT_ON_FAILURE=1 test run-xmltest
+        make CTEST_OUTPUT_ON_FAILURE=1 test
+        if ${with_dtd}; then
+            make run-xmltest
+        fi
 
         lcov -c -d "${capture_dir}" -o "${coverage_info}-test" &>> run.log
         lcov \
@@ -230,9 +243,13 @@ _merge_coverage_info() {
 
 _clean_coverage_info() {
     local coverage_dir="$1"
-    local dir
-    for dir in CMakeFiles examples tests ; do
-        local pattern="*/${dir}/*"
+    local pattern
+    for pattern in \
+            '/usr/**mingw**/include/*' \
+            '*/CMakeFiles/*' \
+            '*/examples/*' \
+            '*/tests/*' \
+        ; do
         (
             set -x
             lcov -q -o "${coverage_dir}/${coverage_info}" -r "${coverage_dir}/${coverage_info}" "${pattern}"
@@ -260,7 +277,7 @@ _show_summary() {
 
 
 _main() {
-    version="$(git describe --tags)"
+    version="$(git describe --tags 2>/dev/null || echo HEAD)"
     coverage_info=coverage.info
 
     local build_dirs=()
@@ -289,6 +306,8 @@ _main() {
     with_unsigned_char=false
     with_libbsd=false
     with_m32=false
+    with_dtd=true
+    with_ge=true
     for with_mingw in true false ; do
         for unicode_enabled in true false ; do
             if ${unicode_enabled} && ! ${with_mingw} ; then
@@ -307,6 +326,8 @@ _main() {
     with_libbsd=true _build_case
     with_unsigned_char=true _build_case
     with_m32=true _build_case
+    with_dtd=false with_ge=true _build_case
+    with_dtd=false with_ge=false _build_case
 
     echo
     echo 'Merging coverage files...'
